%{
/* GTKML - GTK Markup Language
 * Copyright (C) 1998 Scott Dybiec
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

/* GTK and lex/yacc both define TRUE and FALSE */
#define GTKTRUE TRUE
#define GTKFALSE FALSE
#undef TRUE
#undef FALSE

typedef struct Callback
{
   char *event;
   void (*action)();
   GtkWidget *widget;
} Callback;

#include "gtkml.h"

extern int lineno;
extern char *progname;

char linebuf[500];

%}

ws         [ \t]+
comment    #.*
obrace     \{
cbrace     \}
qstring    \"[^\"\n]*[\"]
badqstring \"[^\"\n]*$
id         [a-zA-Z][a-zA-Z0-9]*
number     [0-9]+
nl         \n.*

%%

{ws}               ;
{comment}          ;
{qstring}          { gtkmllval.string = strdup (yytext+1); /* skip open quote */
                     gtkmllval.string[yyleng-2] = '\0'; /* remove close quote */
                     return QSTRING;
                   }
{badqstring}       { gtkmllval.string = strdup(yytext+1); /* skip open quote */
                     warning("Unterminated character string", gtkmllval.string);
                     return QSTRING;
                   }
{obrace}           { return OBRACE; }
{cbrace}           { return CBRACE; }
True               { return TRUE; }
False 	           { return FALSE; }

Window             { return WINDOW; }
Title  	           { return TITLE; }

HorizontalBox      { return HORIZONTALBOX; }
VerticalBox        { return VERTICALBOX; }
BorderWidth        { return BORDERWIDTH; }
Homogeneous        { return HOMOGENEOUS; }
Spacing            { return SPACING; }
Fill               { return FILL; }

Frame              { return FRAME; }
FrameShadow        { return FRAMESHADOW; }
ShadowIn           { return SHADOW_IN; }
ShadowOut          { return SHADOW_OUT; }
ShadowNone         { return SHADOW_NONE; }
ShadowEtchedIn     { return SHADOW_ETCHED_IN; }
ShadowEtchedOut    { return SHADOW_ETCHED_OUT; }

Button             { return BUTTON; }
ToggleButton       { return TOGGLEBUTTON; }
CheckButton        { return CHECKBUTTON; }
Label              { return LABEL; }

ToolTip       	   { return TOOLTIP; }
Entry          	   { return ENTRY; }

NoteBook           { return NOTEBOOK; }
Page               { return PAGE; }
TabPosition        { return TABPOSITION; }
Top                { return TOP; }
Bottom             { return BOTTOM; }
Left               { return LEFT; }
Right              { return RIGHT; }
Center             { return CENTER; }

RadioGroup         { return RADIOGROUP; }
RadioButton        { return RADIOBUTTON; }

List               { return LIST; }
ListItem           { return LISTITEM; }

Widget             { return WIDGET; }

Table              { return TABLE; }
TableDatum         { return TABLEDATUM; }
XPadding           { return XPADDING; }
YPadding           { return YPADDING; }

ComboBox           { return COMBOBOX; }
ComboItem          { return COMBOITEM; }

OnClick            { return ONCLICK; }
OnPress            { return ONPRESS; }
OnRelease          { return ONRELEASE; }
OnEnter            { return ONENTER; }
OnLeave            { return ONLEAVE; }
OnToggle           { return ONTOGGLE; }
Sensitize          { return SENSITIZE; }
Insensitize        { return INSENSITIZE; }
ToggleSensitivity  { return TOGGLESENSITIVITY; }
Show               { return SHOW; }
Hide               { return HIDE; }
ToggleVisibility   { return TOGGLEVISIBILITY; }
SubmitForm         { return SUBMITFORM; }
ResetForm          { return RESETFORM; }
CancelForm         { return CANCELFORM; }

Justify            { return JUSTIFY; }

{id}               { gtkmllval.string = strdup(yytext); 
                     return ID;
                   }
{number}           { gtkmllval.string = strdup(yytext); 
                     return NUMBER;
                   }
{nl}               { strcpy (linebuf, yytext+1);
                     lineno++; 
                     yyless (1);
                   }
.                  { return yytext[0]; }
%%
yywrap()
{
	return 1;
}

gtkmlerror(const char *msg)
{
        fprintf(stderr, "line %d: %s at %s in this line:\n%s\n", 
                        lineno, msg, yytext, linebuf);
}

#if defined(TEST)
main(argc,argv)
int argc;
char **argv;
{

	if (argc > 1) {
		FILE *file;

		file = fopen(argv[1], "r");
		if (!file) {
			fprintf(stderr,"could not open %s\n",argv[1]);
			exit(1);
		}
		yyin = file;
	}
	yylex();
	printf("Done: Read %d lines\n", lineno);
	return 0;
}
#endif
